@@ -31,7 +31,8 @@ logger = logging.getLogger('lda_worker')
 logger.setLevel(logging.INFO)
 
 
-SAVE_DEBUG = 0 # save intermediate models after every SAVE_DEBUG updates (0 for never)
+# periodically save intermediate models after every SAVE_DEBUG updates (0 for never)
+SAVE_DEBUG = 0
 
 
 
@@ -65,7 +66,7 @@ class Worker(object):
     @utils.synchronous('lock_update')
     def processjob(self, job):
         logger.debug("starting to process job #%i" % self.jobsdone)
-        self.model.docEStep(job)
+        self.model.doEstep(job)
         self.jobsdone += 1
         if SAVE_DEBUG and self.jobsdone % SAVE_DEBUG == 0:
             fname = os.path.join(tempfile.gettempdir(), 'lda_worker.pkl')
@@ -79,15 +80,15 @@ class Worker(object):
                     (self.myid, self.jobsdone))
         assert isinstance(self.model.state, ldamodel.LdaState)
         result = self.model.state
-        self.model.state = ldamodel.LdaState()
+        self.model.clear() # free up mem in-between two EM cycles
         return result
 
     
     @utils.synchronous('lock_update')
-    def reset(self, logProbW, alpha):
+    def reset(self, state):
         logger.info("resetting worker #%i" % self.myid)
-        self.model.reset(logProbW)
-        self.model.alpha = alpha
+        self.model.setstate(state)
+        self.model.state.reset(state.sstats)
 
     
     def exit(self):
