@@ -17,6 +17,8 @@ import cPickle
 import itertools
 from functools import wraps # for `synchronous` function lock
 from htmlentitydefs import name2codepoint as n2cp # for `decode_htmlentities`
+import threading, time
+from Queue import Queue
 
 
 
@@ -31,7 +33,7 @@ RE_HTML_ENTITY = re.compile(r'&(#?)(x?)(\w+);', re.UNICODE)
 
 def synchronous(tlockname):
     """
-    A decorator to place an instance based lock around a method.
+    A decorator to place an instance-based lock around a method.
     
     Adapted from http://code.activestate.com/recipes/577105-synchronization-decorator-for-class-methods/
     """
@@ -48,10 +50,9 @@ def synchronous(tlockname):
 
 def deaccent(text):
     """
-    Remove accentuation from the given string.
+    Remove accentuation from the given string. Input text is either a unicode string or utf8 encoded bytestring.
     
-    Input text is either a unicode string or utf8 encoded bytestring. Return
-    input string with accents removed, as unicode.
+    Return input string with accents removed, as unicode.
     
     >>> deaccent("Šéf chomutovských komunistů dostal poštou bílý prášek")
     u'Sef chomutovskych komunistu dostal postou bily prasek'
@@ -63,7 +64,7 @@ def deaccent(text):
     return unicodedata.normalize("NFC", result)
 
 
-def tokenize(text, lowercase = False, deacc = False, errors = "strict", toLower = False, lower = False):
+def tokenize(text, lowercase=False, deacc=False, errors="strict", toLower=False, lower=False):
     """
     Iteratively yield tokens as unicode strings, optionally also lowercasing them 
     and removing accent marks.
@@ -78,7 +79,7 @@ def tokenize(text, lowercase = False, deacc = False, errors = "strict", toLower
     """
     lowercase = lowercase or toLower or lower
     if not isinstance(text, unicode):
-        text = unicode(text, encoding = 'utf8', errors = errors)
+        text = unicode(text, encoding='utf8', errors=errors)
     if lowercase:
         text = text.lower()
     if deacc:
@@ -168,8 +169,8 @@ class FakeDict(object):
         Override the dict.keys() function, which is used to determine the maximum 
         internal id of a corpus = the vocabulary dimensionality.
         
-        HACK: To avoid materializing the whole range(0, self.numTerms), we 
-        return [self.numTerms - 1] only.
+        HACK: To avoid materializing the whole `range(0, self.numTerms)`, this returns 
+        `[self.numTerms - 1]` only.
         """
         return [self.numTerms - 1]
     
@@ -270,13 +271,13 @@ class RepeatCorpus(SaveLoad):
     def __init__(self, corpus, reps):
         """
         Wrap a `corpus` as another corpus of length `reps`. This is achieved by
-        repeating documents from `corpus` over and over again, until requested
-        length is reached. Repetition is done on-the-fly=efficiently, via 
-        itertools. 
+        repeating documents from `corpus` over and over again, until the requested
+        length `len(result)==reps` is reached. Repetition is done 
+        on-the-fly=efficiently, via `itertools`. 
         
         >>> corpus = [[(1, 0.5)], []] # 2 documents
         >>> list(RepeatCorpus(corpus, 5)) # repeat 2.5 times to get 5 documents
-        >>> [[(1, 0.5)], [], [(1, 0.5)], [], [(1, 0.5)]]
+        [[(1, 0.5)], [], [(1, 0.5)], [], [(1, 0.5)]]
 
         """
         self.corpus = corpus
@@ -326,3 +327,84 @@ def decode_htmlentities(text):
         # e.g., ValueError: unichr() arg not in range(0x10000) (narrow Python build)
         return text
 
+
+def chunkize_serial(corpus, chunks):
+    """
+    Split a stream of values into smaller chunks.
+    Each chunk is of length `chunks`, except the last one which may be smaller. 
+    A once-only input stream (`corpus` from a generator) is ok.
+    
+    Chunking is done efficiently with itertools, so that only one chunk at a time
+    is realized in memory (the whole `corpus` may be larger than RAM).
+    
+    >>> for chunk in chunkize(xrange(10), 4): print chunk
+    [0, 1, 2, 3]
+    [4, 5, 6, 7]
+    [8, 9]
+      
+    """
+    chunker = itertools.groupby(enumerate(corpus), key = lambda (docno, doc): docno / chunks)
+    for chunk_no, (key, group) in enumerate(chunker):
+        # return documents as numpy arrays, to save memory with large chunks
+        yield [doc for _, doc in group]
+
+
+
+class InputQueue(threading.Thread):
+    """
+    Help class for threaded `chunkize()`.
+    """
+    def __init__(self, q, corpus, chunks, maxsize):
+        super(InputQueue, self).__init__()
+        self.q = q
+        self.maxsize = maxsize
+        self.corpus = corpus
+        self.chunks = chunks
+    
+    def run(self):
+        import numpy # don't clutter the global namespace with a dependency on numpy
+        for chunk in chunkize_serial(self.corpus, self.chunks):
+            # HACK XXX convert documents to numpy arrays, to save memory.
+            # makes no difference if docs are plain standard list of 2-tuples etc, 
+            # but what if they are more complex, user-defined objects?
+            chunk = [numpy.asarray(doc) for doc in chunk]
+            self.q.put(chunk, block=True)
+#endclass InputQueue
+
+
+def chunkize(corpus, chunks, maxsize=0):
+    """
+    Split a stream of values into smaller chunks.
+    Each chunk is of length `chunks`, except the last one which may be smaller. 
+    A once-only input stream (`corpus` from a generator) is ok, chunking is done
+    efficiently via itertools.
+    
+    If `maxsize > 1`, don't wait idly in between successive chunk `yields`, but 
+    rather keep filling a short queue (of size at most `maxsize`) with forthcoming 
+    chunks in advance. This is realized by starting a separate thread, and is 
+    meant to reduce I/O delays, which can be significant when `corpus` comes 
+    from a slow medium (like harddisk).
+    
+    If `maxsize==0`, don't fool around with threads and simply yield the chunks 
+    via `chunkize_serial()` (no I/O optimizations).
+    
+    >>> for chunk in chunkize(xrange(10), 4): print chunk
+    [0, 1, 2, 3]
+    [4, 5, 6, 7]
+    [8, 9]
+      
+    """
+    assert chunks > 0
+    
+    if maxsize > 0:
+        q = Queue(maxsize=maxsize)
+        thread = InputQueue(q, corpus, chunks, maxsize=maxsize)
+        thread.start()
+        
+        while thread.isAlive() or not q.empty():
+            yield q.get(block=True)
+    else:
+        for chunk in chunkize_serial(corpus, chunks):
+            yield chunk
+
+
